#!/usr/bin/python3

def find_potential_super(path="/etc/tinc/retiolum/hosts"):
  import os
  import re

  needle_addr = re.compile("Address\s*=\s*(.*)")
  needle_port = re.compile("Port\s*=\s*(.*)")
  for f in os.listdir(path):
    try:
      with open(path+"/"+f) as of:
        addrs = []
        port = "655"

        for line in of.readlines():

          addr_found = needle_addr.match(line)
          if addr_found:
            addrs.append(addr_found.group(1))

          port_found = needle_port.match(line)
          if port_found:
            port = port_found.group(1)

        if addrs : yield (f ,[(addr ,int(port)) for addr in addrs])
    except FileNotFoundError as e:
        print("Cannot open hosts directory to be used to find potential supernodes")
        print("Directory used: {}".format(path))
        raise


def try_connect(addr):
  try:
    from socket import socket,AF_INET,SOCK_STREAM
    s = socket(AF_INET,SOCK_STREAM)
    s.settimeout(2)
    s.connect(addr)
    s.settimeout(None)
    s.close()
    return addr
  except Exception as e:
    pass


def check_one_super(ha):
    host,addrs = ha
    valid_addrs = []
    for addr in addrs:
      ret = try_connect(addr)
      if ret: valid_addrs.append(ret)
    if valid_addrs: return (host,valid_addrs)


def check_all_the_super(path):
  from multiprocessing import Pool
  p = Pool(20)
  return filter(None,p.map(check_one_super,find_potential_super(path)))


def main():
  import os
  hostpath=os.environ.get("TINC_HOSTPATH", "/etc/tinc/retiolum/hosts")

  for host,addrs in check_all_the_super(hostpath):
    print("%s %s" %(host,str(addrs)))

if __name__ == "__main__":
    main()

# vim: set expandtab:ts=:sw=2



#!/usr/bin/python
# TODO: Rewrite this shitty piece of software ...
# -*- coding: utf8 -*-

import sys,json,os
""" TODO: Refactoring needed to pull the edges out of the node structures again,
it should be easier to handle both structures"""
DUMP_FILE = os.environ.get("AVAILABILITY_FILE","tinc-availability.json")
hostpath=os.environ.get("TINC_HOSTPATH", "/etc/tinc/retiolum/hosts")

def get_all_nodes():
  return os.listdir(hostpath)

def generate_stats():
  """ Generates availability statistics of the network and nodes
  """
  import json
  jlines = []
  try:
    f = open(DUMP_FILE,'r+')
    for line in f:
      jlines.append(json.loads(line))
    f.close()
  except Exception as e:
    print("Unable to open and parse Availability DB: {} (override with AVAILABILITY_FILE)".format(DUMP_FILE))
    sys.exit(1)

  all_nodes = {}
  for k in get_all_nodes():
    all_nodes[k] = get_node_availability(k,jlines)
  print (json.dumps(all_nodes))

def get_node_availability(name,jlines):
  """ calculates the node availability by reading the generated dump file
  adding together the uptime of the node and returning the time
	parms:
          name - node name
          jlines - list of already parsed dictionaries node archive
  """
  begin = last = current = 0
  uptime = 0
  for stat in jlines:
    if not stat['nodes']:
      continue
    ts = stat['timestamp']
    if not begin:
      begin = last = ts
    current = ts
    if stat['nodes'].get(name,{}).get('to',[]):
      uptime += current - last
    else:
      pass
    last = ts
  all_the_time = last - begin
  try:
    return uptime/ all_the_time
  except:
    return 1

if __name__ == "__main__":
  generate_stats()



#!/usr/bin/python3
# -*- coding: utf8 -*-
import sys,json,os
from .Graph import delete_unused_nodes,resolve_myself
GEODB=os.environ.get("GEODB","GeoLiteCity.dat")

def copy_map():
    from shutil import copy
    from os.path import dirname,join,realpath
    if len(sys.argv) != 2 or sys.argv[1] == "--help" :
        print("usage: {} <destination>".format(sys.argv[0]))
        print("  copies the map.html file to the <destination>")
        sys.exit(1)
    dstdir=sys.argv[1]
    copy(realpath(join(dirname(__file__),'static/map.html')),dstdir)


def add_geo(nodes):
  from pygeoip import GeoIP
  gi = GeoIP(GEODB)

  for k,v in nodes.items():
    try:
      nodes[k].update(gi.record_by_addr(v["external-ip"]))
    except Exception as e:
      sys.stderr.write(str(e))
      sys.stderr.write("Cannot determine GeoData for %s\n"%k)
  return nodes

def add_coords_to_edges(nodes):
  from pygeoip import GeoIP
  gi = GeoIP(GEODB)

  for k,v in nodes.items():
    for i,j in enumerate(v.get("to",[])):
      data=gi.record_by_addr(j["addr"])
      try:
        j["latitude"]=data["latitude"]
        j["longitude"]=data["longitude"]
      except Exception as e: pass

  return nodes

def add_jitter(nodes):
  from random import random
  #add a bit of jitter to all of the coordinates
  max_jitter=0.005
  for k,v in nodes.items():
    jitter_lat= max_jitter -random()*max_jitter*2
    jitter_long= max_jitter -random()*max_jitter*2
    try:
      v["latitude"]= v["latitude"] + jitter_lat
      v["longitude"]= v["longitude"] + jitter_long
      for nodek,node in nodes.items():
        for to in node['to']:
          if to['name'] == k:
            to['latitude'] = v["latitude"]
            to['longitude'] = v["longitude"]
    except Exception as e: pass
  return nodes

def main():
  import json
  try:
      with open(GEODB,'rb') as f: f.read()
  except Exception as e:
      print("cannot open {} (GEODB in env)".format(GEODB))
      print(e)
      sys.exit(1)
  try:
    nodes = add_jitter(add_coords_to_edges(add_geo(resolve_myself(delete_unused_nodes(json.load(sys.stdin))))))
    print (json.dumps(nodes))
  except Exception as e:
      print("cannot parse data received via stdin")
      print(e)

if __name__ == "__main__":
    main()
